// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#pragma once

#include "Common.hpp"			 // for func_t
#include "Context.hpp"		 // for Context
#include "Debug.hpp"			 // for LOGD, LogSubsystem, LogSubsystem::DISP
#include "Fiber.hpp"			 // for Fiber
#include "emper-common.h"	 // for workeraffinity_t

class Runtime;
class ContextManager;

class Dispatcher : public Logger<LogSubsystem::DISP> {
 protected:
	Runtime& runtime;

	void dispatchLoopDoSleep();

	virtual void dispatchLoop() = 0;

	auto getDispatchLoop() -> func_t;

	// The dispatch() method could theoretically be made static in
	// non-debug builds.
	// NOLINTNEXTLINE(readability-convert-member-functions-to-static)
	inline void dispatch(Fiber* fiber) {
		LOGD("executing fiber " << fiber);
		Context::setCurrentFiber(fiber);
		fiber->run();
	}

	static inline auto isRunnable(Fiber* fiber) -> bool { return fiber->setRunnableFalse(); }

	static inline auto getAffinityBuffer(Fiber* fiber) -> workeraffinity_t* {
		return fiber->getAffinityBuffer();
	}

	static inline auto decreaseRefCount(Fiber* fiber) -> unsigned int {
		return fiber->doAtomicDecrRefCount();
	}

	virtual void recycle(Fiber* fiber) { delete fiber; }

	void putRuntimeWorkerToSleep();

 public:
	Dispatcher(Runtime& runtime) : runtime(runtime) {}

	virtual ~Dispatcher() = default;

	static auto getCurrentFiber() -> Fiber& {
		Fiber* fiber = getCurrentFiberPtr();
		return *fiber;
	}

	static auto getCurrentFiberPtr() -> Fiber* { return Context::getCurrentFiber(); }

	static auto isDispatchedControlFlow() -> bool { return getCurrentFiberPtr() != nullptr; }

	friend ContextManager;
};
