// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020 Florian Schmaus
#pragma once

#include <cassert>
#include <functional>
#include <utility>

#include "Common.hpp"
#include "Fiber.hpp"
#include "Runtime.hpp"
#include "SynchronizedFiber.hpp"
#include "io/Future.hpp"

void async(Fiber* fiber) {
	assert(fiber != nullptr);
	Runtime* runtime = Runtime::getRuntime();
	runtime->schedule(*fiber);
}

void async(Fiber::fiber_fun_t function, void* arg) {
	Fiber* fiber = Fiber::from(std::move(function), arg);
	async(fiber);
}

void async(const Fiber::fiber_fun0_t& function) {
	Fiber* fiber = Fiber::from(function);
	async(fiber);
}

void async(Fiber::fiber_fun_t function, void* arg, workeraffinity_t* affinity) {
	Fiber* fiber = Fiber::from(std::move(function), arg, affinity);
	async(fiber);
}

void async(const Fiber::fiber_fun0_t& function, workeraffinity_t* affinity) {
	Fiber* fiber = Fiber::from(function, affinity);
	async(fiber);
}

template <typename S>
void spawn(Fiber::fiber_fun_t function, void* arg, S& semaphore) {
	Fiber* fiber = SynchronizedFiber::from(function, arg, semaphore);
	async(fiber);
}

template <typename S>
void spawn(Fiber::fiber_fun0_t function, S& semaphore) {
	Fiber* fiber = SynchronizedFiber::from(function, semaphore);
	async(fiber);
}

template <typename S>
void spawn(Fiber::fiber_fun_t function, void* arg, workeraffinity_t* affinity, S& semaphore) {
	Fiber* fiber = SynchronizedFiber::from(function, arg, affinity, semaphore);
	async(fiber);
}

template <typename S>
void spawn(Fiber::fiber_fun0_t function, workeraffinity_t* affinity, S& semaphore) {
	Fiber* fiber = SynchronizedFiber::from(function, affinity, semaphore);
	async(fiber);
}

namespace emper {
void yield() {
	Runtime* runtime = Runtime::getRuntime();
	runtime->yield();
}

auto sleep(unsigned int seconds) -> bool {
	if constexpr (!emper::IO) {
		DIE_MSG("sleep requires emper::io");
	}

	emper::io::AlarmFuture::Timespec ts = {.tv_sec = seconds, .tv_nsec = 0};
	emper::io::AlarmFuture alarm(ts);
	int32_t res = alarm.submitAndWait();

	return res == -ETIME;
}
}	 // namespace emper
