// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020-2021 Florian Schmaus, Florian Fischer
#pragma once

#include <boost/thread/shared_mutex.hpp>
#include <queue>

namespace lib::adt {

template <typename I>
class BoostSharedMutexUnboundedQueue {
 private:
	boost::shared_mutex queue_mutex;

	std::queue<I*> queue;

 public:
	void enqueue(I* item) {
		boost::unique_lock<boost::shared_mutex> lock(queue_mutex);
		queue.push(item);
	}

	template <class InputIt>
	void insert(InputIt begin, InputIt end) {
		boost::unique_lock<boost::shared_mutex> lock(queue_mutex);
		for (; begin != end; ++begin) {
			queue.push(*begin);
		}
	}

	auto dequeue() -> I* {
		boost::upgrade_lock<boost::shared_mutex> rlock(queue_mutex);
		if (queue.empty()) {
			return nullptr;
		}

		boost::upgrade_to_unique_lock<boost::shared_mutex> wlock(rlock);
		I* res = queue.front();
		queue.pop();
		return res;
	}
};
}	 // namespace lib::adt
