// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020-2021 Florian Schmaus, Florian Fischer
#pragma once

#include <mutex>
#include <queue>

namespace lib::adt {

template <typename I>
class MutexUnboundedQueue {
 private:
	std::mutex queue_mutex;

	std::queue<I*> queue;

 public:
	void enqueue(I* item) {
		std::lock_guard<std::mutex> lock(queue_mutex);
		queue.push(item);
	}

	template <class InputIt>
	void insert(InputIt begin, InputIt end) {
		std::lock_guard<std::mutex> lock(queue_mutex);
		for (; begin != end; ++begin) {
			queue.push(*begin);
		}
	}

	auto dequeue() -> I* {
		std::lock_guard<std::mutex> lock(queue_mutex);
		if (queue.empty()) {
			return nullptr;
		}
		I* res = queue.front();
		queue.pop();
		return res;
	}
};
}	 // namespace lib::adt
