// SPDX-License-Identifier: LGPL-3.0-or-later
// Copyright © 2020-2021 Florian Schmaus, Florian Fischer
#pragma once

#include <queue>
#include <shared_mutex>

namespace lib::adt {

template <typename I>
class SharedMutexUnboundedQueue {
 private:
	std::shared_mutex queue_mutex;

	std::queue<I*> queue;

 public:
	void enqueue(I* item) {
		std::unique_lock lock(queue_mutex);
		queue.push(item);
	}

	template <class InputIt>
	void insert(InputIt begin, InputIt end) {
		std::unique_lock lock(queue_mutex);
		for (; begin != end; ++begin) {
			queue.push(*begin);
		}
	}

	auto dequeue() -> I* {
		{
			std::shared_lock lock(queue_mutex);
			if (queue.empty()) {
				return nullptr;
			}
		}

		std::unique_lock lock(queue_mutex);
		if (queue.empty()) {
			return nullptr;
		}
		I* res = queue.front();
		queue.pop();
		return res;
	}
};
}	 // namespace lib::adt
